@@ -41,6 +41,7 @@ def _wait_for_bcache_setup(bcache_dev):
             break
 
 class KbdZRAMTestCase(unittest.TestCase):
+    @skip_on(("fedora", "27"), reason="zram module (un)loading is broken on ")
     def setUp(self):
         self.addCleanup(self._clean_up)
         self._loaded_zram_module = False
@@ -50,7 +51,7 @@ class KbdZRAMTestCase(unittest.TestCase):
         if self._loaded_zram_module:
             os.system("rmmod zram")
 
-class KbdZRAMDevicesTestCase(unittest.TestCase):
+class KbdZRAMDevicesTestCase(KbdZRAMTestCase):
     @unittest.skipUnless(_can_load_zram(), "cannot load the 'zram' module")
     @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
     def test_create_destroy_devices(self):
@@ -515,7 +516,7 @@ class KbdUnloadTest(unittest.TestCase):
         # unload all plugins first
         self.assertTrue(BlockDev.reinit([], True, None))
 
-        with fake_path():
+        with fake_path(all_but="make-bcache"):
             with self.assertRaises(GLib.GError):
                 BlockDev.reinit(None, True, None)
 
